; Testprogramm zum Lesen und Beschreiben eines seriellen EEPROMs 24C04 auf dem seriellen 80C552-Board.
; Leitung SDA an Port 3.4, SCL an Port 3.5, WP an Port 3.3
; Die Eingnge PRE, E1, E2 sind an Masse.

#cpu = 80C552	; @16 MHz

#Const Lesen = 1010 0001b		;Block 0
#Const Lesen1 = 1010 0011b		;Block 1
#Const Schreiben = 1010 0000b	;Block 0
#Const Schreiben1 = 1010 0010b	;Block 1
#Const Adresse = 0001 0000b		;EEPROM Adresse in Block 0 oder 1, kann gendert werden.
#Bit Data = P3.4, CLK = P3.5, WP = P3.3

;Es wird das Wort Hallo an die Adresse 10h des EEPROMs geschrieben.
Reset:
	clr WP	; whlt Page-Write (ohne das gehts nicht)
	Startbedingung erzeugen
		mov a, #Schreiben
	8 Bits senden
	Acknowledge
		mov a, #Adresse
	8 Bits senden
	Acknowledge
		mov a, #'H'
	8 Bits senden
	Acknowledge
		mov a, #'a'
	8 Bits senden
	Acknowledge
		mov a, #'l'
	8 Bits senden
	Acknowledge
		mov a, #'l'
	8 Bits senden
	Acknowledge
		mov a, #'o'
	8 Bits senden
	Acknowledge
	Stoppbedingung erzeugen	;Startet Brennvorgang

;Hier wird auf Beendigung des Brennvorgangs gewartet:
	loop
		Startbedingung erzeugen
			mov a, #Schreiben
		8 Bits senden
		Acknowledge
	until not bit c

;Auszulesende Adresse schreiben:
		mov a, #Adresse
	8 Bits senden
	Acknowledge

;EEPROM auf Lesen umstellen
	Startbedingung erzeugen
		mov a, #Lesen
	8 Bits senden
	Acknowledge

	8 Bits lesen
		cpl a
		mov P1, a		; An Port 5 wird der ASCII-Code des Buchstabens H (= 0100 1000b) angezeigt.

	Stoppbedingung erzeugen
end

Startbedingung erzeugen:
	setb CLK
		>5s
	clr Data
		>5s
	clr CLK
ret

Stoppbedingung erzeugen:
	clr Data
		>5s
	setb CLK
		>5s
	setb Data
ret

Acknowledge:
	setb Data
		>5s
	setb CLK
		>5s
	mov c,Data		; c = 0 ist erfolgreiche Operation.
		>5s
	clr CLK
		>5s
ret

8 Bits senden:		; Daten sind in a
	for r2 = #8
		rlc a
		mov Data, c
		Takt
	next		
ret

8 Bits lesen:		; Daten werden in den Akku geschrieben.
	for r2 = #8
		setb CLK
			>5s
		mov c, Data
		rlc a
		clr CLK
			>5s
	next		
ret

Takt:
	>5s			; Low-Pegel
	setb CLK
	>5s			; High-Pegel
	clr CLK
ret

>5s:		; dient der Zeitverzgerung (@16 MHz)
;	nop
ret