; Liest den kompletten Inhalt des seriellen EEPROMs 24C04 und schreibt ihn in das externe RAM des Controllers.
; Leitung SDA an Port 3.4, SCL an Port 3.5, WP an Port 3.3
; Die Eingnge PRE, E1, E2 an Masse

#cpu = 80C552	; @16 MHz

#Const Lesen = 1010 0001b		;Block 0
#Const Lesen1 = 1010 0011b		;Block 1
#Const Schreiben = 1010 0000b	;Block 0
#Const Schreiben1 = 1010 0010b	;Block 1
#Bit Data = P3.4, CLK = P3.5, WP = P3.3

Reset:
	clr WP	; Write Protect
	Startbedingung erzeugen
		mov a, #Schreiben
	Akku senden
	Acknowledge lesen

;Auszulesende Adresse schreiben:
		mov a, #0
	Akku senden
	Acknowledge lesen

;EEPROM auf Lesen umstellen:
	Startbedingung erzeugen
		mov a, #Lesen
	Akku senden
	Acknowledge lesen

	clr a
	P2 = a	: r0 = a	; Page 0 im XRAM
	for r3 = a		; 256 Bytes = Page 0 lesen
		8 Bits lesen
		movx @r0, a: inc r0
		Acknowledge senden
	next

	inc P2			;Page 1 im XRAM
	for r3 = #0	;256 Bytes = Page 1 lesen
		8 Bits lesen
		movx @r0, a: inc r0
		Acknowledge senden
	next

	8 Bits lesen		; und verwerfen, da vor dem STOP kein ACK erfolgen darf.
	setb CLK		; = No ACK
		>5s
	clr CLK
		>5s
	Stoppbedingung erzeugen
	mov P1, #AAh	; zeigt Programmende an.
end
;	***	ENDE	***

Startbedingung erzeugen:
	setb CLK
		>5s
	clr Data
		>5s
	clr CLK
ret

Stoppbedingung erzeugen:
	clr Data
		>5s
	setb CLK
		>5s
	setb Data
ret

Acknowledge lesen:		; empfangen, 24c04 to Controller
	setb Data
		>5s
	setb CLK
		>5s
	mov c, Data		; c = 0 ist erfolgreiche Operation.
		>5s
	clr CLK
		>5s
ret

Acknowledge senden:	; Controller to 24c04
	clr Data
		>5s
	setb CLK
		>5s
	clr CLK
	setb Data
		>5s
ret

Akku senden:		; Daten sind in a
	for r2 = #8
		rlc a
		mov Data, c
		Takt
	next		
ret

8 Bits lesen:		;Daten werden in den Akku geschrieben.
	for r2 = #8
		setb CLK
			>5s
		mov c, Data
		rlc a
		clr CLK
			>5s
	next		
ret

Takt:
	>5s			;Low-Pegel
	setb CLK
	>5s			;High-Pegel
	clr CLK
ret

>5s:		;Dient der Zeitverzgerung (@16 MHz)
	nop
ret