; Software fr 4x20 Display 80C552-Board, verwendet Image im IRAM von 80h bis CFh

#cpu = 80C552	; @16 MHz
#Byte 1kHz Teiler, Cursor

ajmp Initialisierung

Timer 0:	; Timer 0 Interrupt
	djnz 1kHz Teiler, Reti
		mov 1kHz Teiler, #10
		acall Display
Reti:
reti

Initialisierung:
	clr a	; Variablenbereich lschen
	r0 = #1
	loop
		@r0 = a: inc r0
	until r0 = #0
	mov 1kHz Teiler, #10
	orl TMOD, #2	; Timer 0 im 8-Bit Autoreload-Modus. 
	; Die berlauffrequenz des Timer 0 betrgt 10000 Hz, die Periodendauer 0,1 ms.
	mov TL0, #7Bh
	mov TH0, #7Bh	; Reloadwert
	setb TR0	; Timer 0 luft.

	; Interrupts
	setb ET0		; Timer 0 Interrupt freigeben
	setb EA		; Globale Interruptfreigabe
	acall DispIni

	r0 = #80h + 1: Textausgabe
	r0 = #80h + 23: Textausgabe
	r0 = #80h + 45: Textausgabe
	r0 = #80h + 67: Textausgabe
end
; * * * Hauptprogramm Ende * * *

Textausgabe:
	r7 = #8		; Tabellenoffset
	loop
		a = r7: movc a, @a+pc
		if a = #0 then ret
		@r0 = a: inc r0: inc r7
	end loop
DB 'Probetext ', 0

; Display-Routinen: 
;	DispIni(): erzeugt 2 Zeilen ohne Cursor.
;	DispCLR: lscht IRAM-Image
;	DispHex(a): gibt Akkuinhalt in Hex aus.
;	DispBCD(a): gibt Akkuinhalt dezimal aus.
;	DispVBCD(a): gibt Akkuinhalt signiert dezimal aus.
;	DispBCDb(a): unterdrckt fhrende Nullen
;	DispDByte(dlow,dhigh): stellt DoubleByte-Wert dezimal dar.
;	DispVDByte(dlow,dhigh): zeigt Doppelbyte signiert an
;	Display@cursor(a,Cursor): beschreibt IRAM-Image
;	DISPLAY(a,Daten): gibt Befehle oder Daten an Anzeige.

#Byte SubHigh, SubLow, Zahl, dlow, dhigh, IRAM Pointer
#Bit Daten, Adresse gesetzt

DispIni:		;Interrupt muss freigegeben sein.
	IRAM Pointer = #81h		;Startposition
	clr Daten				;Befehle
	r0 = #81h: a = #38h: @r0 = a	 ;2 Zeilen
	inc r0: a = #1: @r0 = a	;CLR, Home
	inc r0: a = #6: @r0 = a 	;Increment
	inc r0: a = #Ch: @r0 = a	;Cursor aus
	loop					;Interrupt muss vorher freigegeben sein.
		a = #85h		;Warten, bis Daten geschrieben sind.
	until a = IRAM Pointer	;Wird im Interrupt (Display) verndert.
	acall DispCharDef
	DispCLR
	IRAM Pointer = #80h		;BasisAdresse im Controller-RAM setzen
	setb Daten		;Umstellung auf Daten fr Display
ret

DispCharDef:		; definiert eigene Zeichen
	IRAM Pointer = #81h		;Startposition
	r0 = #81h: @r0 = #40h		; Adresse 0
	loop: a = IRAM Pointer: until a = #82h
	setb Daten		;Umstellung auf Daten fr Display
	mov dptr, #Displaydata g: acall Displaydata
	mov dptr, #Displaydata j
Displaydata:
	r0 = #81h: IRAM Pointer = #81h
	loop
		clr a: movc a, @a+dptr: @r0 = a
		inc dptr: inc r0
	until r0 = #89h
	loop: a = IRAM Pointer: until a = #89h
ret

Displaydata g:
	DB 0, 0, 1111b, 10001b, 10001b, 1111b, 1, 1110b
Displaydata j:
	DB 10b, 0, 110b, 10b, 10b, 10b, 10010b, 1100b

DispCLR:
	a = #' ': r0 = #80h
	loop
		@r0 = a: inc r0
	until r0 = #D0h
ret

DispHex:			;Wert in a, bentigt r7
	push acc
	swap a			;High-Byte verwenden
		INT(a/10)*7+48+a
	pop acc
	INT(a/10)*7+48+a:
		anl a, #0Fh	;Low-Byte isolieren
		mov r7, a		;In a steht das Low-Nibble
		mov b, #10	;Bin-Hex Umwandlung:
		div ab			;Rechnung:
		mov b, #7		;	INT(a / 10) * 7 + 48 + a
		mul ab   		;	ergibt ASCII 0 - 9 bzw. Ah - Fh
		add a, #48: add a, r7: Display@cursor
ret

DispVBCD:			;Wertbergabe in a
	push acc
	if not bit acc.7 then
		mov a, #'+': Display@cursor
		pop acc
	else
		mov a, # '-': Display@cursor
		pop acc
		cpl a: inc a	;Zweierkomplement bilden
	end if
DispBCD:			;Wertbergabe in a
	b = #64h: div ab: add a, #30h: Display@cursor	;ASCII-Korrektur, 100er
	a = b: b = #10: div ab: add a, #30h: Display@cursor	;10er
	a = b: add a, #30h: Display@cursor			;1er Stelle
ret

DispBCDb:			; Fhrende Nullen unterdrcken, Wertbergabe in ACC
	b = #64h: div ab: add a, #30h		;ASCII-Korrektur, 100er
	if a = #'0' then: a = #' ': end if
	Display@cursor
	a = b: b = #10: div ab: add a, #30h	;10er
	if a = #'0' then: a = #' ': end if
	Display@cursor
	a = b: add a, # 30h: Display@cursor	;1er Stelle
ret

DispVDByte:		;Zeigt Doppelbyte signiert an
;	dec Cursor	; ?
	a = dhigh
	if bit acc.7 then	;minus
		mov a, # '-': Display@cursor
		a = dlow: cpl a: add a, #1: dlow = a		;Zweierkomplement bilden
		a = dhigh: cpl a: addc a, #0: dhigh = a	;eventuelles Carry addieren
	else					;plus
		mov a, # ' ': Display@cursor	;Leerzeichen bei Plus
	end if
DispDByte:			;Bentigt R7,R6, Wertbergabe in dlow und dhigh.
	For r6 = #3		;Zahl der Durchlufe.
		a = r6			;Teilt durch 10.000, 1000, 100
		rl a				;x 2
		add a, #9		;Tabellenoffset
		movc a, @a+pc: SubHigh = a: a = r6: rl a	;x 2
		add a, #3		;neues Tabellenoffset
		movc a,@a+pc: SubLow = a: sjmp DispD0
	;Tabelle:
	DB	00, 64h, 03, E8h, 27h, 10h
	DispD0:
		Zahl = #0: clr  c
		loop
			a = dlow: subb a, SubLow	;Subtrahiert 2710h(=10.000d)
			r7 = a			;03E8h(=1000d), 0064h=(100d) bis berlauf
			a = dhigh: subb a, SubHigh
			if not bit c then		;Am Carry ist berlauf erkennbar.
				dhigh = a: dlow = r7: inc  Zahl
		end loop
			end if
		a = Zahl: add a, #30h: Display@cursor	;Gibt den Wert zur Anzeige.
	Next
	a = dlow: b = #10: div ab: add a, #30h: Display@cursor	;10er
	a = b: add a, #30h: Display@cursor		;1er Stelle
ret

Display@cursor:	;bergabe in a, ndert r0 und Cursor
	r0 = Cursor: orl 0, #80h	; 0 = r0
	@r0 = a: inc Cursor
	if r0 = #CFh then	;Display Grenze
		mov Cursor, #0	;von vorne beginnen
	end if
ret

DISPLAY:		; wird periodisch mit 1kHz im Timer-Interrupt aufgerufen
	push PSW: push acc	; Einstellungen retten
	mov dptr, #F000h		; aktiviert Y7, ist Displayadresse
	r2 = IRAM Pointer: clr a
	if r2 = #80h then: a = r2
	elseif r2 = #94h then: a = #80h + 40h
	elseif r2 = #A8h then: a = #80h + 20
	elseif r2 = #BCh then: a = #80h + 40h + 20
	elseif r2 = #D0h then: a = #80h: IRAM Pointer = a: end if
	if a > # 0 then	;neue Zeile steht an
		jbc Adresse gesetzt, Display1
		movx @dptr, a	;Cursorposition in Anzeige setzen.
		setb Adresse gesetzt
	else
	Display1:
		r0 = IRAM Pointer: a = @r0  ;Image aus IRAM lesen
		if bit Daten then	;Daten = 0 -> Befehle; Daten = 1 -> Daten
			inc dptr
			if a = #'0' then: a = #'O'	; Standardzeichen ersetzen
			elseif a = # 'g' then: clr a
			elseif a = # 'p' then: a = #F0h
			elseif a = # 'q' then: a = #F1h
			elseif a = # 'j' then: a = #1
			elseif a = # '' then: a = #E1h
			elseif a = # '' then: a = #F5h
			elseif a = # '' then: a = #EFh
			elseif a = # '' then: a = #DFh
			end if
		end if
		movx @dptr, a		;Befehle/Daten an Anzeige senden.
		inc IRAM Pointer
	end if
	pop acc: pop PSW
ret
